@@ -1,8 +1,8 @@
 
-__version__ = "2.0.1"
+__version__ = "2.2"
 docs = r"https://crocodile.readthedocs.io/en/latest/"
 git = r"https://github.com/thisismygitrepo/crocodile"
 release_notes = """
 
-Library name is now crocodile
+Fixed deep learning problems
 """
